
/*---------------------------------------------------------------------------*/
/* Display */
/*---------------------------------------------------------------------------*/

#include <avr/io.h>
#include <util/delay.h> 

#include "string.h"
#include "stdlib.h"

#include "def.h"
//#include "itoa.h"



#define  LCD_E_1        PORTC |= BIT2   //H: Enable
#define  LCD_E_0        PORTC &= ~BIT2  //L: Disable
#define  LCD_DATA       PORTC
#define  LCD_DATA_READ  PINC
#define  DDR_WRITE      {DDRC = 0xff;}    //Daten-Port auf Ausgang 
#define  DDR_READ       {DDRC = 0x0f; LCD_DATA &=0x0f; } //Daten-Port auf Eingang mit Pull-Up aus
#define  LCD_RS_1scr    {PORTC |=  BIT3;} //H: Data to screen
#define  LCD_RS_0reg    {PORTC &= ~BIT3;} //L: Data to register 
#define  LCD_RW_1r      {DDR_READ; PORTC |=  BIT1; }  //H: Read from LCD
#define  LCD_RW_0w      {PORTC &= ~BIT1; DDR_WRITE;}  //L: Write to LCD


// LCD Katalog
enum {
  LCD_BLANK,
  LCD_NORMAL1,
  LCD_NORMAL2,
  LCD_TIMER1,
  LCD_TIMER2,
  LCD_OFEN1,
  LCD_OFEN2,
  LCD_SCHALTUHR1,
  LCD_SCHALTUHR2,
  LCD_HELLO,
  LCD_SERVICE1_1,
  LCD_SERVICE1_2,
  LCD_BEEPER_AUS,
  LCD_BEEPER_EIN,
  LCD_FLASH1, //Alle des abgelaufenen Timers hell
  LCD_FLASH2, //Alle des abgelaufenen Timers hell

  LCD_ListenEnde};

//Textkonstanten
char *ofenTexte[]={
    "-AUS-",   //0
    "Licht",
    " Um  ",   //2
    "Um+Uh",   
    "Um+Oh",   //4
    " Oh  ",      
    " Uh  ",   //6
    "Oh+Uh",   
    "Grill"    //8
  };








// ----------------------------------------------------------------------------
// value to text converter 
// ----------------------------------------------------------------------------

char * btoa0x(char * buffer, unsigned char val){   // len=5: sprintf(buffer, "%02x",val);
U8 h,l;
  h = (val >>4) & 0x0f;
  l = val & 0x0f;
  if (h>9){
    buffer[0] = h +55;
  }else{
    buffer[0] = h +48;
  }    
  if (l>9){
    buffer[1] = l +55;
  }else{
    buffer[1] = l +48;
  }    
  buffer[2]=0;
  return buffer+2;
}
//-----------------------------------------------------------------------------

char * stoa(char *buffer, char * txt){
  while ((*buffer++ = *txt++));
  return buffer-1;
}


//-----------------------------------------------------------------------------
//Druckt Zahl mit anhaengendem Text in Puffer
//Zahl wird rechtsbuendig mit <len> Stellen ausgegeben und mit Spaces aufgefuellt
//Ist die Zahl zu lang, werden len "#" ausgegeben
//Negativen Zahlen wird ein "-" vorangestellt
//Liefert Zeiger auf den abschliessenden '\0'.

//Dieses Macro dient zum definieren der Funktion fuer verschiedenen Integer-Datentypen
#define INT_STR2STR(name, vartyp) char * name(char * buffer, vartyp val, int len, char * txt ){ \
  char neg; \
  int i; \
    if (len<1) return buffer; \
    if (val < 0){ \
      val=0-val; \
      neg=1; \
    }else{ \
      neg=0; \
    }   \
    i=len; \
    do{ \
      buffer[--i] = (val % 10) + '0'; \
      val /= 10; \
    }while (val && (i>neg)); \
    if (val){ \
      for (i=0; i<len; buffer[i++] = '#'); \
      i=0;\
    }\
    if (i && neg){ \
      buffer[--i] = '-'; \
    } \
    while (i){ \
      buffer[--i] = ' '; \
    } \
    buffer+=len;   \
    while ((*buffer++ = *txt++)); \
    return buffer-1; \
  } 
  
//Druckt Zahl mit anhaengendem Text in Puffer
//Zahl wird rechtsbuendig mit <len> Stellen ausgegeben und mit 0 aufgefuellt
//Ist die Zahl zu lang, werden len "#" ausgegeben
//Negativen Zahlen wird ein "-" vorangestellt
//Liefert Zeiger auf den abschliessenden '\0'.

//Dieses Macro dient zum definieren der Funktion fuer verschiedenen Integer-Datentypen
#define INT_STR2STR0(name, vartyp) char * name(char * buffer, vartyp val, int len, char * txt ){ \
  char neg; \
  int i; \
    if (len<1) return buffer; \
    if (val < 0){ \
      val=0-val; \
      neg=1; \
    }else{ \
      neg=0; \
    }   \
    i=len; \
    do{ \
      buffer[--i] = (val % 10) + '0'; \
      val /= 10; \
    }while (val && (i>neg)); \
    if (val){ \
      for (i=0; i<len; buffer[i++] = '#'); \
      i=0;\
    }\
    while (i>neg){ \
      buffer[--i] = '0'; \
    } \
    if (neg){ \
      buffer[--i] = '-'; \
    } \
    buffer+=len;   \
    while ((*buffer++ = *txt++)); \
    return buffer-1; \
  } 
  


//(int val, int len, char * txt, char * buffer)
INT_STR2STR(isTOa,int);
INT_STR2STR0(isTOa0,int);
INT_STR2STR(uisTOa,unsigned int);
INT_STR2STR0(uisTOa0,unsigned int);
INT_STR2STR(lsTOa,long);
INT_STR2STR(ulsTOa,unsigned long);





// ----------------------------------------------------------------------------
//   LCD functions
// ----------------------------------------------------------------------------



// ----------------------------------------------------------------------------
void checkBusy (void){
// ----------------------------------------------------------------------------
//ACHTUNG: Hat im 4-bit Modus einfach nicht funktioniert
U8 busy;

	LCD_RS_0reg;   //Befehl
	_delay_us(1);
	LCD_RW_1r;   //Lesen
	_delay_us(1);
	do{
    //1. Nibble abholen (darin ist das busy Flag)
		LCD_E_1;
  	_delay_us(2);
		busy = LCD_DATA_READ & BIT7; //Busy flag
		LCD_E_0;
  	_delay_us(1);

    //2. Nibble abholen um Zugriff vollstndig zu machen
		LCD_E_1;
  	_delay_us(1);
		LCD_E_0;
  	_delay_us(1);

	}while(busy);

	LCD_RW_0w; //Schreiben

}

// ---------------------------------------------------------------------------
/*! Send nibble to display.
    Depending on the value of LCD_RS the byte is written to display(LCD_RS=1) or register (LCD_RS=0)

   \param b is the nibble to be send. 
*/// -------------------------------------------------------------------------
void lcdWriteNibbleH(U8 b) { //b: only Bits 4..7 are used
      LCD_E_1;
      _delay_us(1);
      LCD_DATA = (LCD_DATA & 0x0F) | (b & 0xf0); //b in obere 4 Bits schreiben
      _delay_us(2);
      LCD_E_0;
      _delay_us(2);
}
void lcdWriteNibbleL(U8 b) { //b: only Bits 0..3 are used
      LCD_E_1;
      _delay_us(1);
      LCD_DATA = (LCD_DATA & 0x0F) | ((b<<4) & 0xf0); //b in obere 4 Bits schreiben
      _delay_us(2);
      LCD_E_0;
      _delay_us(2);
}

/*! Send byte to display.
    Depending on the value of LCD_RS the byte is written to display(LCD_RS=1) or register (LCD_RS=0)

   \param b is the byte to be send.
   \note the BUSY-Flag isn't checked (since we can't read from display with R/W hard-wired to GND)
*/// -------------------------------------------------------------------------
void lcdWrite(char b) { 
// ---------------------------------------------------------------------------
  lcdWriteNibbleH(b);    //Sendet high Nibble
  lcdWriteNibbleL(b);    //Sendet low Nibble

  // CAUTION: we do not check the BUSY-Flag (since we can't read from display)
  // CAUTION: so for HOME and CLEAR commands the user has to add 2ms delay!
  //checkBusy();
 }


// ---------------------------------------------------------------------------
void cmd2Lcd(U8 cmd) {
// ---------------------------------------------------------------------------
  LCD_RS_0reg;
  lcdWrite(cmd);
  _delay_us(10);
  LCD_RS_1scr;
  _delay_us(10);
}
// ---------------------------------------------------------------------------
/*! Clear LCD and set cursor home
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------

void clearLcd(void) {
  cmd2Lcd(0x01);
  //_delay_ms(7); wird durch verlassen von ProcessLCDPanel getimed
}
// ---------------------------------------------------------------------------
/*! Set cursor home
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
void home1Lcd(void) {
  cmd2Lcd(0x02);
}
// ---------------------------------------------------------------------------
/*! Set cursor at start of Line 2
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
void home2Lcd(void) {
  cmd2Lcd(128+64);
}
// ---------------------------------------------------------------------------
/*! Define user characters
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
static const U8 fontDef[16]={
//Was nicht im Datenblkatt steht: im 4Bit-Modus muss mind. 1 Bit je Zeile gesetzt sein (ich nehme Bit7)
      // #1
    0b10001000,
    0b10010100,
    0b10001000,
    0b10000011,
    0b10000100,
    0b10000100,
    0b10000011,
    0x80,
      // #2
    0x80,
    0x80,
    0x80,
    0x80,
    0x80,
    0x80,
    0b10010000,
    0x80,
  };
void defineUserChar(void) {
  U8 i;
    LCD_RS_0reg;
    lcdWrite(64+8); //Address in CharacterGenerator-Ram
    LCD_RS_1scr;


    for (i=0; i<16; i++){ //8 characters * 8 bytes for each character
      lcdWrite(fontDef[i]);
	}

    LCD_RS_0reg;
    lcdWrite(128); //Set DDRAM Address
    LCD_RS_1scr;
}


//----- String senden ----------------------------------------------------------
void sendStringToLCD(char *str){
  U8 i;

  //Zeichen fuer Zeichen senden (ohne abschliessende '\0')
  i = 0;
  LCD_RS_1scr;
  _delay_us(1);

  while (str[i] !=0)
  {
    lcdWrite(str[i]);
    i++;
    // mehr als 20 Zeichen werden nie an das Display gesendet
    if (i > 20){
      break;
    }
  }
}


// ---------------------------------------------------------------------------
/*! Initialize display (2 Lines, 8 Bit, no shift, increment)
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
void lcdInit(void) {

  LCD_DATA=0;
  LCD_RW_0w;   //write
  LCD_RS_0reg; //register
  LCD_E_0;  

  PORTD &=~BIT6; //Display power off (Hardware-Reset des Displays)
  _delay_ms(400); //Warte, bis CPU-Versorgung stabil
  PORTD |=BIT6; //Display power on

  _delay_ms(150);  //warte bis Display bereit

  //Make sure to be in 8 Bit mode
  lcdWriteNibbleH(0x30); 
  _delay_us(500);
  lcdWriteNibbleH(0x30); 
  _delay_us(500);
  lcdWriteNibbleH(0x30); 
  _delay_us(500);


  //Switch to 4bit
  lcdWriteNibbleH(0x20); 
  _delay_us(500);


// Wegen dem schneren  Zeichen (0xD2) htte ich gerne den Zeichensatz "Western European"
//Leider geht der nicht. Also bleibt'S beim Englisch/Japanischen.
  lcdWrite(0x28); //4bit mode, Zeichensatz Englisch/Japanisch
//  lcdWrite(0x29); //Zeichensatz Western European  <---- geht nicht (Display dunkel)
  //lcdWrite(0x2A); //4bit mode,  Zeichensatz Russisch/Englisch  <--- geht
//  _delay_us(500);


  lcdWrite(0x08); //Display OFF

  lcdWrite(0x06); //entry mode set increment cursor by 1 not shifting display

  lcdWrite(0x17); //Character mode and internel power on


 
  lcdWrite(0x01); //clear display
  _delay_ms(7);

  lcdWrite(0x02); //Home
 



#if 0
  lcdWrite(0x40); //CG Address=0
  _delay_ms(7);
  lcdWrite(0x80); //DD Address=8
  _delay_ms(7);
#endif

  lcdWrite(0x0c); //Display ON
  LCD_RS_1scr; //screen
//  _delay_ms(7);

  defineUserChar();



  //sendStringToLCD("12345");
//  cmd2Lcd(128+20); //Zeile2
 // sendStringToLCD("ABCDE");
//  sendStringToLCD("ABCDEFGHIJKLMNOPQRS");


}


// -----------------------------------------------------------------------------
void textbyCatalog(char * zeile, U8 catalogNr)
{
  int n; //Anzahl ausgegebener Zeichen
  int intVal1;
  int intVal2;
  int intVal3;
  char *p;
  U8 i;
  U8 c;
  
  switch (catalogNr)
  {

    case LCD_BLANK:  //Leerzeile
      zeile[0]=0;
      n=1;
      break;
/*      123456789 123456789  
        ====+====+====+====+
         15+             15
         15+  14:23:44   15
        ====+====+====+====+
   
    
        ====+====+====+====+
         15+ Ul+Uh 180  15   Ofen:Istwerte
         15+ Kern   77  15
        ====+====+====+====+
*/    
    
    
    
    case LCD_NORMAL1:{    //Schalterstufen hinten
      if (g.iCounter[EBENE_OFEN][1]){
        p=stoa(zeile, " ");
        p=isTOa(p,g.iCounter[EBENE_NORMAL][0],2,"  ");
        p=stoa(p, ofenTexte[g.iCounter[EBENE_OFEN][1]]);
        p=stoa(p, " ");
        p=isTOa(p,g.iOfentempCelsius,3,""); 
        *p++ = GRADZEICHEN;
        p=stoa(p, "  ");
        p=isTOa(p,g.iCounter[EBENE_NORMAL][1],2,"");
      }else{ //Kein Ofen.
        p=stoa(zeile, " ");
        //                                       123456789 1234
        p=isTOa(p,g.iCounter[EBENE_NORMAL][0],2,"              ");
        p=isTOa(p,g.iCounter[EBENE_NORMAL][1],2,"");
      }
      if (g.bAussenH){
        zeile[3]='+';
      }
      break;
    }

/*
         14+ Kern   77  14 
        ====+====+====+====+

         14+  13:45:12   14 
        ====+====+====+====+

         14+ 1h 32m 29s  14 
        ====+====+====+====+

         14+ 0h  2m 07s  14 
        ====+====+====+====+
    */

    case LCD_NORMAL2:{  //Schalterstufen Vorn, Ggf. Kerntemp, nchster Timer oder Uhrzeit
      if (g.iCounter[EBENE_OFEN][1] && g.mitKern){
        p=stoa(zeile, " ");
        p=isTOa(p,g.iCounter[EBENE_NORMAL][2],2,"  Kern   ");
        p=isTOa(p,g.iKerntempCelsius,2,"");
        *p++ = GRADZEICHEN; 
        p=stoa(p, "  ");

        p=isTOa(p,g.iCounter[EBENE_NORMAL][3],2,"");
      }else{
        //Ist ein Timer gesetzt?
        if (g.iNaechsterTimer_s < 32400){ //Timer gesetzt. Zeige Zeit bis zum Piepsen.
          p=stoa(zeile, " ");
          //                                       12
          p=isTOa(p,g.iCounter[EBENE_NORMAL][2],2,"  ");

          intVal1 = g.iNaechsterTimer_s / 3600;
          intVal2 = (g.iNaechsterTimer_s % 3600)/60;
          intVal3 = g.iNaechsterTimer_s %60;
          if (intVal1 >0){
            p=isTOa(p,intVal1,1,"h ");
            p=isTOa(p,intVal2,2,"m ");
        }else{     //123
            p=stoa(p, "   ");
			if (intVal2 > 0){
              p=isTOa(p,intVal2,2,"m ");
			}else{
              p=stoa(p, "    ");
			}
          }
//          p=isTOa(p,intVal2,2,"m ");
          p=isTOa(p,intVal3,2,"s  ");
          p=isTOa(p,g.iCounter[EBENE_NORMAL][3],2,"");
        
        }else{//Kein Timer gesetzt. Zeige Uhrzeit bzw. Einschaltdauer
          p=stoa(zeile, " ");
          //                                       123
          p=isTOa(p,g.iCounter[EBENE_NORMAL][2],2,"   ");
          p=isTOa(p,g.bStunde,2,":");
          p=isTOa0(p,g.bMinute,2,":");
          p=isTOa0(p,g.bSekunde,2,"   ");
          p=isTOa(p,g.iCounter[EBENE_NORMAL][3],2,"");
        }
      }
      if (g.bDcf77Empfang){
        zeile[0]=DCF77DOT;
      }  
      if (g.bAussenV){
        zeile[3]='+';
      }
      break;
    }
/*
         123456789 123456789  
         ====+====+====+====+
          --:-- Timer 112:30 
         114:30        --:-- 
         ====+====+====+====+
*/
    case LCD_TIMER1:{  //Zeiten in Minuten:Sekunden
      if (g.iCounter[EBENE_TIMER][0]){
        intVal1 = g.iCounter[EBENE_TIMER][0] / 60;
        intVal2 = g.iCounter[EBENE_TIMER][0] % 60;
        p=isTOa(zeile,intVal1,3,":");
        p=isTOa0(p,intVal2,2," Timer ");
      }else{
        p=stoa(zeile, " --:-- Timer ");
      }
      if (g.iCounter[EBENE_TIMER][1]){
        intVal1 = g.iCounter[EBENE_TIMER][1] / 60;
        intVal2 = g.iCounter[EBENE_TIMER][1] % 60;
        p=isTOa(p,intVal1,3,":");
        p=isTOa0(p,intVal2,2,"");
      }else{
        p=stoa(p, " --:--");
      }
      break;
    }

    case LCD_TIMER2:{  //Zeiten in Minuten:Sekunden
      if (g.iCounter[EBENE_TIMER][2]){
        intVal1 = g.iCounter[EBENE_TIMER][2] / 60;
        intVal2 = g.iCounter[EBENE_TIMER][2] % 60;
        p=isTOa(zeile,intVal1,3,":");
        p=isTOa0(p,intVal2,2,"       ");
      }else{
        p=stoa(zeile, " --:--       ");
      }
      if (g.iCounter[EBENE_TIMER][3]){
        intVal1 = g.iCounter[EBENE_TIMER][3] / 60;
        intVal2 = g.iCounter[EBENE_TIMER][3] % 60;
        p=isTOa(p,intVal1,3,":");
        p=isTOa0(p,intVal2,2,"");
      }else{
        p=stoa(p, " --:--");
      }
      break;
    }
/*    
        ====+====+====+====+
        180   Ofen   Ul+Oh    Ofen+Kern Sollwerte
        Kern 77      1:30h
        ====+====+====+====+
        180   Ofen   Ul+Oh    Ofen+Kern Sollwerte
        Kern 77     1:30:59
        ====+====+====+====+
        180   Ofen  AUS 
        Kern 99      --:--      Countdown Timer (Stunden/Minuten) Druck schaltet auf Schaltuhr
        ====+====+====+====+     
        180   Ofen  AUS 
        Kern 99     TIMER!     Wenn Dauer(Schaltuhr)>0      
        ====+====+====+====+     
*/    
    case LCD_OFEN1: { 
      p=isTOa(zeile,g.iCounter[EBENE_OFEN][0],3,"");
      *p++ = GRADZEICHEN;
      p=stoa(p, "   Ofen   ");
      p=stoa(p, ofenTexte[g.iCounter[EBENE_OFEN][1]]);
      break;
    }
    case LCD_OFEN2: { 
      p=stoa(zeile, "Kern ");
      p=isTOa(p,g.iCounter[EBENE_OFEN][2],2,"");
      *p++ = GRADZEICHEN;  
      p=stoa(p, "     ");
      if (g.iSchaltuhrDauer>0){
        p=stoa(p, "Timer!");
      }else{
        if (g.iCounter[EBENE_OFEN][3] >0){
          intVal1 = g.iCounter[EBENE_OFEN][3] / 3600;
          intVal2 = (g.iCounter[EBENE_OFEN][3] % 3600)/60;
          intVal3 = g.iCounter[EBENE_OFEN][3] %60;
          p=isTOa(p,intVal1,1,":");
          p=isTOa0(p,intVal2,2,":");
          p=isTOa0(p,intVal3,2,"");
        }else{
          p=stoa(p, " --:-- ");
        }
      }
      break;
    }
/*
        ====+====+====+====+
        Dauer   1:00       
        Ende   12:30  
        ====+====+====+====+
        Dauer  --:--  
        Ende   13:00        
        ====+====+====+====+
*/    
    case LCD_SCHALTUHR1: { 
      p=stoa(zeile, "Dauer  ");
      if ((g.iCounter[EBENE_SCHALTUHR][0] > 0) || (g.iCounter[EBENE_SCHALTUHR][1]>0)){
        p=isTOa(p,g.iCounter[EBENE_SCHALTUHR][0],2,":");
        p=isTOa0(p,g.iCounter[EBENE_SCHALTUHR][1],2,"    ");
      }else{
        p=stoa(p, "--:--    ");
      }
      break;
    }
    case LCD_SCHALTUHR2: { 
        p=stoa(zeile, "Ende   ");
        p=isTOa(p,g.iCounter[EBENE_SCHALTUHR][2],2,":");
        p=isTOa0(p,g.iCounter[EBENE_SCHALTUHR][3],2,"    ");
      break;
    }
    
    
    case LCD_HELLO: { //Begrung beim Start
                   //123456789 123456789
      p=stoa(zeile, " Koch was Leckeres!");
      break;
    }

    
    case LCD_SERVICE1_1: { //Istwerte Roh
                   //123456789 123456789
      p=stoa(zeile, "IST ROH Platine:");
      p=isTOa(p,g.iPlatinentempRoh,3,"");  
      break;
    }
    case LCD_SERVICE1_2: { //Istwerte Roh
      //123456789 123456789
      //Kern:123 Ofen:123
      p=stoa(zeile, "Kern:");
      p=isTOa(p,g.iKerntempRoh,3," Ofen:");  
      p=isTOa(p,g.iOfentempRoh,3,"");  
      break;
    }
    case LCD_BEEPER_AUS: { //Piezo ein/aus
                   //123456789 123456789
      p=stoa(zeile, "   Timer-Ton AUS   ");
      break;
    }
    case LCD_BEEPER_EIN: { //Piezo ein/aus
                   //123456789 123456789
      p=stoa(zeile, "   Timer-Ton EIN   ");
      break;
    }
      
    case LCD_FLASH1: { //Alle Pixel des abgelaufenen Timers hell (Bei Backofen: alle 4)
      p=zeile;
      c = (g.bAbgelaufeneTimer & BIT0) ? 0xff : 0x20;//Alle Pixel hell oder Leerzeichen
      for (i=0; i<10; i++) *p++ = c; 
      c = (g.bAbgelaufeneTimer & BIT1) ? 0xff : 0x20;//Alle Pixel hell oder Leerzeichen
      for (i=10; i<19; i++) *p++ = c; 
      break;
    }
    case LCD_FLASH2: { //Alle Pixel des abgelaufenen Timers hell
      p=zeile;
      c = (g.bAbgelaufeneTimer & BIT2) ? 0xff : 0x20;//Alle Pixel hell oder Leerzeichen
      for (i=0; i<10; i++) *p++ = c; 
      c = (g.bAbgelaufeneTimer & BIT3) ? 0xff : 0x20;//Alle Pixel hell oder Leerzeichen
      for (i=10; i<19; i++) *p++ = c; 
      break;
    }
    
    default:
      zeile[0]=0;
      break;


  }

  if (strlen(zeile) > 20){
    //Huaaaa! Fataler Fehler
    strcpy(zeile,"internal Error 20");
  }
}

// ------------------------------------------------------------------------------
// Write current data to panel
void refreshPanel(void){
static char tmpZeile[30];
  
//  clearLcd();
  
  //---1.Zeile---
  home1Lcd();
  switch (g.bEbene){
    case EBENE_NORMAL:        textbyCatalog(tmpZeile, LCD_NORMAL1);           break;
    case EBENE_TIMER:         textbyCatalog(tmpZeile, LCD_TIMER1);            break;
    case EBENE_OFEN:          textbyCatalog(tmpZeile, LCD_OFEN1);             break;
    case EBENE_SCHALTUHR:     textbyCatalog(tmpZeile, LCD_SCHALTUHR1);        break;
    case EBENE_SERVICE:       textbyCatalog(tmpZeile, LCD_SERVICE1_1);        break;
  }
  if (g.bZeigeTonAus){
    if (g.bBeeperEnabled){
      textbyCatalog(tmpZeile, LCD_BEEPER_EIN);
    }else{
      textbyCatalog(tmpZeile, LCD_BEEPER_AUS);
    }

  }
  if (g.bBlinken && g.blinkphase){
    textbyCatalog(tmpZeile, LCD_FLASH1);
  }
#if 0 // Debug DCF
  if (DI_DCF77_BIT){
    tmpZeile[0]='|';
  }else{
    tmpZeile[0]='_';
  }
#endif
  sendStringToLCD(tmpZeile);

  //---2.Zeile---
  home2Lcd();
  switch (g.bEbene){
    case EBENE_NORMAL:        textbyCatalog(tmpZeile, LCD_NORMAL2);           break;
    case EBENE_TIMER:         textbyCatalog(tmpZeile, LCD_TIMER2);            break;
    case EBENE_OFEN:          textbyCatalog(tmpZeile, LCD_OFEN2);             break;
    case EBENE_SCHALTUHR:     textbyCatalog(tmpZeile, LCD_SCHALTUHR2);        break;
    case EBENE_SERVICE:       textbyCatalog(tmpZeile, LCD_SERVICE1_2);        break;
  }
  if (g.bBlinken && g.blinkphase){
    textbyCatalog(tmpZeile, LCD_FLASH2);
  }

// Debuggen der Tasteneingnge
#if 0
U8 i;
U8 x;
  x=PINB;  
  for (i=0; i<4; i++){
    if (x & 1) tmpZeile[i] = 'B'; else  tmpZeile[i] = '0';
    x>>=1;
  }
  tmpZeile[5]=' ';
  tmpZeile[6]=' ';
  x=PIND;  
  for (i=6; i<10; i++){
    if (x & 1) tmpZeile[i] = 'D'; else  tmpZeile[i] = '0';
    x>>=1;
  }
  tmpZeile[10]=' ';
  tmpZeile[11]=' ';
  tmpZeile[12]=' ';

#endif
  sendStringToLCD(tmpZeile);
}


//-----------------------------------------------------------------------------
void ProcessLCDPanel(void){
static U8 init=1;
static U8 bLicht=0;
static U8 n=0;
static U8 bMerkEbene=-1;  

char tmpZeile[30];

  if (init){
    init=0;

    lcdInit();
  //  LICHT=1;

    //Bei Start kurz eine Begrung anzeigen
    textbyCatalog(tmpZeile, LCD_HELLO);
    sendStringToLCD(tmpZeile);
    _delay_ms(1000);
  }

  //--- Hier gehts dann immmer rund ------------------------------------------

  if (bMerkEbene!= g.bEbene){
    bMerkEbene= g.bEbene;
    g.bRefreshDisplay=1;
    clearLcd();  //Das dauert 7ms - also erst beim nchsten Aufruf weitermachen
 	return;
  }
  
  if (g.bRefreshDisplay){
    g.bRefreshDisplay=0;
    refreshPanel();
  }



  //Licht blinken lassen?
  if (g.bBlinken){
    n++;
    if (n>5){
      n=0;
      if (bLicht){
//        LICHT=0;  //Licht aus
        bLicht=0;
      }else{
//        LICHT=1;  //Licht an
        bLicht=1;
      }
    }
  }else{
    if (g.iCounter[EBENE_NORMAL][0] || 
        g.iCounter[EBENE_NORMAL][1] ||
        g.iCounter[EBENE_NORMAL][2] ||
        g.iCounter[EBENE_NORMAL][3] ||
        g.iCounter[EBENE_TIMER] [0] ||
        g.iCounter[EBENE_TIMER] [1] ||
        g.iCounter[EBENE_TIMER] [2] ||
        g.iCounter[EBENE_TIMER] [3] ||
        g.iCounter[EBENE_OFEN]  [1] ||
        (g.bLichtCounter < 10) ||
        (g.bEbene!=EBENE_NORMAL)
       )
    {
//      LICHT=1;
    }else{
//      LICHT=0;
    }
    bLicht=1;
  }
}
//EOF

