/* DCF-77 Decoder */
/*
Whrend jeder Minute werden die Nummern von Minute, Stunde, Tag, Wochentag, 
Monat und Jahr BCD-kodiert durch Impulsmodulation der Sekundenmarken bertragen. 
Dieses "Telegramm" gilt jeweils fr die folgende Minute. 
Dabei entsprechen Sekundenmarken mit einer Dauer von 0,1 s der binren Null 
und solche mit einer Dauer von 0,2 s der binren Eins. Die Zuordnung der einzelnen 
Sekundenmarken auf die bertragene Zeitinformation zeigt das Kodierschema 


0.              Minutenbeginn (immer LOW)
1. - 14.        Reserviert, keine Bedeutung
15.             Reserveantenne aktiv
16.             Umstellung von Sommer- auf Winterzeit, oder umgekehrt
17.             Sommerzeit aktiv
18.             Winterzeit aktiv
19.             Ankndigung Schaltsekunde
20.             Zeitbeginn (Immer High)
21. - 27.       Minute 1, 2, 4, 8, 10, 20, 40
28.             Prfbit Minute
29. - 34.       Stunde 1, 2, 4, 8, 10, 20
35.             Prfbit Stunde
36. - 41.       Tag 1, 2, 4, 8, 10, 20
42. - 44.       Wochentag 1, 2, 4
45. - 49.       Monat 1, 2, 4, 8, 10
50. - 57.       Jahr 1, 2, 4, 8, 10, 20, 40, 80
58.             Prfbit Datum
59.             fehlt zur Erkennung des Minutenanfangs

(siehe auch http://www.heret.de/funkuhr/ptb.htm)

Die drei __Prfbits__ P1, P2 und P3 ergnzen jeweils die vorhergehenden Informationswrter 
(7 Bits fr die Minute, 6 Bits fr die Stunden und 22 Bits fr das Datum, einschlielich 
der Nummer des Wochentages) auf eine __gerade__ Anzahl von Einsen.


Die Zeitmarken Nr. 17 und 18 zeigen an, auf welches Zeitsystem sich die ausgesandte 
Zeitinformation bezieht. Bei Aussendung der MEZ wird die Sekundenmarke Nr. 18 auf 0,2 s
verlngert; die Sekundenmarke Nr. 17 hat eine Dauer von 0,1 s. Bei der Aussendung der 
MESZ ist es umgekehrt.

Vor einem bergang von MEZ nach MESZ oder zurck wird auerdem jeweils eine Stunde 
lang die Sekundenmarke Nr. 16 als verlngerte Marke (0,2 s) ausgesendet. Diese 
Verlngerung beginnt beim bergang von MEZ auf MESZ (von MESZ nach MEZ) um 
01.00.16 Uhr MEZ (2.00.16 Uhr MESZ) und endet um 01.59.16 Uhr MEZ (02.59.16 Uhr MESZ).

Die Sekundenmarke Nr. 19 kndigt eine Schaltsekunde an. Sie wird dann ebenfalls 
eine Stunde lang vor Einfhrung der Schaltsekunde als verlngerte Marke (0,2 s) 
ausgesendet. Schaltsekunden werden weltweit zum gleichen Zeitpunkt in die koordinierte 
Weltzeitskala UTC eingefhrt, vorzugsweise am Ende der letzen Stunde des 31. Dezember 
oder des 30. Juni. Dies bedeutet, da Schaltsekunden in der gesetzlichen Zeit der 
Bundesrepublik Deutschland eine Sekunde vor 1 Uhr MEZ am 1. Januar oder vor 
2 Uhr MESZ am 1. Juli eingeschoben werden. Bei einer Schaltsekunde am 1. Januar (1. Juli)
beginnt daher die Verlngerung der Sekundenmarke Nr. 19 um 00.00.19 Uhr MEZ (01.00.19 Uhr MESZ)
und endet um 00.59.19 Uhr MEZ (01.59.19 Uhr MESZ).

Beim Einfgen einer Schaltsekunde hat die zugehrige Minute eine Dauer von 61 Sekunden,
und die der Marke 01.00.00 Uhr MEZ (02.00.00 Uhr MESZ) vorhergehende 59. Sekundenmarke 
wird mit einer Dauer von 0,1 s ausgesendet. Die zur eingefgten 60. Sekunde gehrige 
Marke wird weggelassen (keine Trgerabsenkung).

*/

#include <util/delay.h> 
#include <avr/io.h>
#include <avr/interrupt.h>
#include "def.h"


//-----------------------------------------------------------------------------

void task_dcf77(void){
 
static U8  init=1;  
static I8  sTiefpass=0;
static U8  bSekunde=0;    //0..59
static U8  bSteigend=0;  //TRUE: Tiefpass bewegt sich von 0 aufwrts (Nahe Sekundenanfang)
static U8  bBit[60];  
static U8   bChecksum;  
static U8  bStunde;
static U8  bMinute;  
static U8  bSchnellstart=1;
static U16 iTic=0;        //ms seit letztem Absenkungsbeginn
U8 i;

  if (init){
    init=0;
    for (i=0; i<60; i++){
      bBit[i]=0;
    }
  }  



  iTic+=5;
  if (iTic>3000){ //Kein Signal
    iTic=3000;
  }
  //Eingang abfragen und filtern
  if (DI_DCF77_BIT==DCF77_BIT_GESETZT){ //Signalabsenkung fr 100ms oder 200ms
    sTiefpass++;
    if (sTiefpass>40) sTiefpass=40;  //auf 200ms begrenzen
  }else{
    sTiefpass--;
    if (sTiefpass<0){
      sTiefpass=0;
      bSteigend=1;
    }
  }
  //Minutenanfang erkennen
  if (iTic > 1500){ //Minutensignal: mehr als 1s keine Absenkung
    bSekunde=59;
  }
  else
  //Signallnge bestimmen und im Bitfeld eintragen
  if (150==iTic){ //150ms  Tiefpass ist entweder +20-10=10 oder +20+10=30
    if (sTiefpass > 19){
      bBit[bSekunde]=1;
    }else{
      bBit[bSekunde]=0;
    }
  }
  //Sekundenanfang erkennen
  if ((12==sTiefpass) && bSteigend){ //60ms
    bSteigend=0;
    iTic=60;
    bSekunde++;
    g.bDcf77Empfang=!g.bDcf77Empfang;
    if (bSekunde>59) bSekunde=0;
    if (1==bSekunde){
      for (i=2; i<59; i++){  //Bits als "nicht empfangen" markieren
        bBit[i]=9;
      }
	}    
    //Auswertung zu bestimmten Zeiten
    if (0==bSekunde){
      bChecksum=bBit[21]+bBit[22]+bBit[23]+bBit[24]+bBit[25]+bBit[26]+bBit[27]+bBit[28]; //Minute
      for (i=2; i<59; i++){
	    if (bBit[i] > 1){ //Bit wurde nicht empfangen
		  bChecksum=1; //ungltige Checksumme
		}
	  }
      if (0==(bChecksum % 2)){ 
        bChecksum=bBit[29]+bBit[30]+bBit[31]+bBit[32]+bBit[33]+bBit[34]+bBit[35];        //Stunde
        if (0==(bChecksum % 2)){
          if (bBit[20]){ //Startbit; immer gesetzt
            bMinute=bBit[21]+bBit[22]*2+bBit[23]*4+bBit[24]*8+bBit[25]*10+bBit[26]*20+bBit[27]*40;
            bStunde=bBit[29]+bBit[30]*2+bBit[31]*4+bBit[32]*8+bBit[33]*10+bBit[34]*20;
            if (bSchnellstart){
              bSchnellstart=0;
              //bernehme Werte
              g.bStunde=bStunde;
              g.bMinute=bMinute;
              g.bSekunde=bSekunde;
            }else{

              bChecksum=0;
              for (i=36; i<59; i++){
                bChecksum+=bBit[i];
              }
              if (0==(bChecksum % 2)){
                //bernehme Werte
                g.bStunde=bStunde;
                g.bMinute=bMinute;
                g.bSekunde=bSekunde;
              }
            }
          }
        }     
      }
    }
  }
}
//EOF

